# -*- coding: utf-8 -*-
""" db core

    Core tables for the application.
    
"""
import locale
from datetime import datetime
from gluon.tools import PluginManager

dbui = local_import('plugin_dbui')

#-------------------------------------------------------------------------------
#
# plugin configuration parameters
#
#-------------------------------------------------------------------------------
plugins = PluginManager()
#plugins.dbui.app_css = None                    # relative path, i.e static/myapp.css or dir or list
#plugins.dbui.app_js = None                     # relative path static/src/my.js or dir or list
#plugins.dbui.app_libmin = None                 # relative path static/myapp-min.js or dir or list
#plugins.dbui.app_script = None                 # relative path static/myapp.js
plugins.dbui.app_script_dir = 'static/scripts'  # relative directory static/scripts

#plugins.dbui.dbui_libmin='static/plugin_dbui/dbui-debug.js'

#-------------------------------------------------------------------------------
#
# set the languages for the application
#
#-------------------------------------------------------------------------------
T.set_current_languages('en', 'en-gb', 'en-us')     # mother tongue
T.force('fr-fr')                                    # user language
T.lazy = False                                      # immediate translation

locale.setlocale(locale.LC_ALL, "fr_FR.UTF-8")

#-------------------------------------------------------------------------------
#
# set constants
#
#-------------------------------------------------------------------------------
undef = T(dbui.UNDEF)
undef_id = dbui.UNDEF_ID
year = datetime.now().year

#-------------------------------------------------------------------------------
#
# core tables
#
#-------------------------------------------------------------------------------
db.define_table("categories",
    Field("code", "string", notnull=True, unique=True),
    Field("definition", "text", notnull=True),
    migrate="categories.table")

db.categories.code.requires = IS_MATCH('^[A-Z]+$')

db.define_table("collaborations",
    Field("collaboration", "string", notnull=True, unique=True),
    migrate="collaborations.table")

db.define_table("countries",
    Field("country", "string", notnull=True, unique=True),
    Field("abbreviation", "string", notnull=True),
    migrate="countries.table")

db.define_table("projects",
    Field("project", "string", notnull=True, unique=True),
    migrate="projects.table")

db.define_table("publishers",
    Field("publisher", "string", notnull=True, unique=True),
    Field("abbreviation", "string", notnull=True),
    migrate="publishers.table")

db.define_table("reports",
    Field("type", "string", notnull=True, unique=True),
    migrate="reports.table")

db.define_table("teams",
    Field("team", "string", notnull=True, unique=True),
    migrate="teams.table")

#-------------------------------------------------------------------------------
#
# Publications table
#
#-------------------------------------------------------------------------------
tp_authors = \
"""List of authors separated by commma: J. Doe, P.-Y. Smith
For large collaboration the first author followed by et al.: J. Doe et al."""

tp_authors_cppm = \
"""List of authors separated by comma: P.-Y. Doe, J. Smith"""

tp_report_numbers = \
"""List of report identifier separated by comma: 
LHCb-PROC-2008-04, arXiv:0906.1516 """
 
tp_speaker = \
"""The name of the speaker: P.-Y. Smith"""

db.define_table("publications",
    Field("title", "text", notnull=True),
    Field("authors", "string", notnull=True, comment=tp_authors),
    Field("id_collaborations", db.collaborations, default=undef_id, label="Collaboration"),
    Field("id_publishers", db.publishers, default=undef_id,  label="Publisher"),
    Field("year", "integer", notnull=True, default=year, requires=IS_INT_IN_RANGE(1900, year+1)),
    Field("doi", "string"),
    Field("volume", "integer", requires=IS_EMPTY_OR(IS_INT_IN_RANGE(1, None))),
    Field("first_page", "integer", requires=IS_EMPTY_OR(IS_INT_IN_RANGE(1, None))),
    Field("last_page", "integer", requires=IS_EMPTY_OR(IS_INT_IN_RANGE(1, None))),
    Field("e_print", "string", requires=IS_EMPTY_OR(IS_URL())),
    Field("conference_title", "text", label='Title'),
    Field("conference_url", "string", label='url', requires=IS_EMPTY_OR(IS_URL())),
    Field("conference_start", "date", label='Start date'),
    Field("conference_end", "date", label='End date'),
    Field("conference_town", "string", label='Town'),
    Field("id_countries", db.countries, default=undef_id, label='Country'),
    Field("conference_speaker", "string", label='Speaker', comment=tp_speaker),
    Field("report_numbers", "string", comment=tp_report_numbers),
    Field("id_reports", db.reports, default=undef_id, label="Report type"),
    Field("authors_cppm", "text", notnull=True, comment=tp_authors_cppm),
    Field("id_teams", db.teams, default=undef_id, label='Team'),
    Field("id_projects", db.projects, default=undef_id, label='Projects'),
    Field("id_categories", db.categories, default=undef_id, label='AERES'),
    migrate="publications.table")

db.publications.id_categories.requires = \
IS_IN_DB(db, 'categories.id', 'categories.code')

db.publications.id_collaborations.requires = \
IS_IN_DB(db, 'collaborations.id', 'collaborations.collaboration')

db.publications.id_countries.requires = \
IS_IN_DB(db, 'countries.id', 'countries.country')

db.publications.id_publishers.requires = \
IS_IN_DB(db, 'publishers.id', 'publishers.abbreviation')

db.publications.id_projects.requires = \
IS_IN_DB(db, 'projects.id', 'projects.project')

db.publications.id_reports.requires = \
IS_IN_DB(db, 'reports.id', 'reports.type')

db.publications.id_teams.requires = \
IS_IN_DB(db, 'teams.id', 'teams.team')
