/**
 * The paging bottom bar plugin
 * The ptype of this component is pGridPaging.
 *  
 *  
 * NOTE: the number of row load in the grid at the first time
 * is defined by the grid when loading the store.
 * 
 * @version
 * 
 */
Ext.namespace('App.grid');

App.grid.Paging = Ext.extend(Object, {

    /**
     * Private parameter identifying the type of plugin
     */
    ptype: 'pGridPaging',

    /**
     * Private attribute for internationalisation
     */
    textSlider: 'Rows per page',
    
    /**
     * Plugin initialisation
     */
    init: function (grid) {

        var bbar;
        
        // create the paging bar
        bbar = new Ext.PagingToolbar({store: grid.store});

        // replace the bottomtool bar by the pagingtool bar
        grid.bottomToolbar = bbar;

        // add slider
        bbar.add(               
            '-',
            this.textSlider, 
            {
                xtype: 'slider',
                plugins: new Ext.slider.Tip(),
                listeners: {
                    changecomplete: this.onChangePageSize,
                    scope: bbar
                },
                minValue: 1,
                width: 100
            }, '-'
         );       
        
        // Initialise the parameters of the paging and slider widgets.
        // Depends on the status of the store.
        // Initialisation is only performed once.
        if (grid.store.getTotalCount() > 0) {
            
            this.onInit.call(grid, grid.store);
        
        } else {
        
            grid.store.on('load', this.onInit, grid, {single: true});
        
        }
        
        // update the slider parameter after a write action
        // which destroy and create records
        grid.store.on('write', this.onWrite, grid);
        
        // reset paging and slide parameter on a grid reset
        grid.on('resetgrid', this.onReset, grid);
        
    },
    
    /**
     * Handler to change the number of rows per page
     * the scope is the bottom tool bar
     */
    onChangePageSize: function (slider, newValue, thumb) {
        var bbar = this;
        bbar.pageSize = newValue;
        bbar.moveFirst();
    },
    
    /**
     * Handler to initialise the number of rows per page and the number of page
     * the scope is the grid
     */
    onInit: function (store, records, options) {

        var bbar, 
            grid = this,
            nRecords=store.getTotalCount(),
            nRows = store.getCount(), 
            slider;

        bbar = grid.getBottomToolbar();
        bbar.pageSize = nRows;
        bbar.moveFirst();
        
        slider = bbar.findByType('slider')[0];
        slider.setMaxValue(nRecords);
        slider.setValue(nRows);
    },
    
    /**
     * Handler to reset the paging parameters
     * The scope is the grid 
     */
    onReset: function() {
        var bbar,
            grid = this;

        bbar = grid.getBottomToolbar();
        bbar.pageSize = grid.nRows;
        bbar.moveFirst();
        
        slider = bbar.findByType('slider')[0];        
        slider.setMaxValue(grid.store.getTotalCount());
        slider.setValue(grid.nRows);

    },
    /**
     * Handler to update the slider/bottomToolBar parameters after a store write action.
     * The latter create or destroy records in the store.
     * The scope is the bottom grid
     */
    onWrite: function () {
        
        var bbar,
            grid = this,
            slider;

        bbar = grid.getBottomToolbar();
        bbar.pageSize = grid.store.getCount();
        
        slider = bbar.findByType('slider')[0];
        slider.setMaxValue(grid.store.getTotalCount());
        slider.setValue(grid.store.getCount());
        
    }
});

Ext.preg('pGridPaging', App.grid.Paging);
